package coi.jbi.client;

import coi.relationMonitor.RelationMonitorService;
import kaos.core.csi.CSIFactory;
import kaos.core.csi.registration.AgentConfiguration;
import kaos.core.csi.registration.Registration;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.policy.guard.GuardInstantiationException;
import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.KAoSGuard;
import org.infospherics.jbi.client.*;
import org.infospherics.jbi.client.PublisherSequence;
import org.infospherics.jbi.client.exception.*;
import org.infospherics.jbi.client.typemgt.MetadataRepository;

import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * This interface provides the methods that are necessary for establishing and managing connections to the JBI Platform. It provides a means for
 * authenticating client access to a platform and contains the primary interface methods that may be used by the client when interacting with a
 * specific platform. <br>
 * Once authenticated the connection interface is utilized to:
 * <ul>
 * <li>Create information objects
 * <li>Create publish, subscribe and/or query sequence interfaces
 * <li>Establish access to the metadata schema repository by creating a MetadataRepository interface
 * </ul>
 *
 * @version CAPI 1.5
 */
public class Connection implements org.infospherics.jbi.client.Connection
{
	private org.infospherics.jbi.client.Connection 	_connection;
    protected RelationMonitorService _relationMonitorService = null;
    protected AgentConfiguration myAgentConfig = null;
    protected KAoSAgentDescription myAgentDesc = null;

	public Connection(org.infospherics.jbi.client.Connection connection)
	{
		_connection = connection;
	}

   /**
   * This method is used as the authentication function in which client applications seeking access to the JBI platform submit credentials to the
   * platform specific security algorithms. This method <b>MUST </b> be called and authentication performed before any other method in this interface
   * can be successfully utilized. The access privileges defined for a client are controlled by the information management staff and are platform
   * specific.
   *
   * @param clientRep
   *          An XML document that contains the client's description of itself to be used for authentication. The structure and semantics of the
   *          credentials are define by the specific platform implementation.
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @since CAPI 1.0
   */
  public void authenticate(String clientRep) throws PlatformFailureException, PermissionDeniedException
  {
	  // ---------- Register in KAoS first --------------

      // String userID = getUserId(clientRep);
	  registerWithKAoS();

      KAoSGuard guard = (KAoSGuard)GuardRetriever.getGuard();
      _relationMonitorService = (RelationMonitorService)guard.getKAoSExtensionComponents("RelationMonitor", RelationMonitorService.RELATION_MONITOR_SERVICE_URL);

      // Do the original
	  _connection.authenticate(clientRep);
  }

  /**
   * This method is used to establishing a connect to the JBI Platform and is executed following the authentication method. Once the connection is
   * establish the client application can create:
   * <ul>
   * <li>Information objects
   * <li>Publish, subscribe and/or query sequences
   * <li>A MetadataRepository interface
   * </ul>
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @since CAPI 1.0 <br>
   *        CAPI 1.5: Modified to return platform UID instead of void
   */
  public void connect( ) throws PlatformFailureException, PermissionDeniedException
  {
	  _connection.connect();
  }

  /**
   * This method is used to disconnect from the JBI Platform. This method will cause the platform to release used resources (kill threads, etc.)
   * without destroying the Connection itself. Successfully execution of any methods associated with existing sequence or repository interfaces is
   * temporarily suspended until the <b>connect </b> method is used to reestablish the client to platform Connection.
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @since CAPI 1.0
   */
  public void disconnect( ) throws PlatformFailureException, PermissionDeniedException
  {
      // deregister from KAoS
        try
        {
            Registration registration = CSIFactory.getRegistration();
            registration.deregisterEntity(myAgentDesc);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            System.out.println("\n\n\t*** UNABLE TO CONNECT TO KAoS!  Will continue without KAoS services!");

            return;
        }
        // disconnect from JBI
        _connection.disconnect();
  }

  /**
   * Returns the connection uid <br>
   *
   * @return <b>connectionUID </b> the connectionUID generated by the server.
   * @throws PlatformFailureException
   * @throws PermissionDeniedException
   */
  public String getConnectionUID( ) throws PlatformFailureException, PermissionDeniedException
  {
	  return _connection.getConnectionUID();
  }

  /**
   * This method is used to retrieve the connection name that was declared using the ConnectionManager interface when the connection was originally
   * created.
   *
   * @return <b>connectionName </b> the client provided name of the connection.
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see ConnectionManager
   *
   * @since CAPI 1.0
   */
  public String getConnectionName( ) throws PlatformFailureException, PermissionDeniedException
  {
	  return _connection.getConnectionName();
  }

  /**
   * This method is used to retrieve the descriptor that describes the particular platform implementation. The descriptor was declared using the
   * ConnectionManager interface when the connection was created. The descriptor is meant to be an XLM document that describes the particular platform
   * implementation. In the future, this document will allow clients to potentially connect to a number possibly disparate platform implementations
   * within the same client. Currently an legal string can be used to describe the particular platform.
   *
   * @return <b>connectionDescriptor </b> XML document that describes the particular platform implementation.
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see ConnectionManager
   *
   * @since CAPI 1.0
   */
  public String getConnectionDescriptor( ) throws PlatformFailureException, PermissionDeniedException
  {
	  return _connection.getConnectionDescriptor();
  }

  /**
   * Creates a PublisherSequence that will subsequently be used to publish information objects to the JBI platform. The sequence is created
   * identifying the specific information object type that will subsequently be published. Unique type name and version strings are supplied to
   * identify the information object type. A schema must exist in the metadata schema repository that corresponds to the declared information object type.
   *
   * @param typeName
   *          the information object type name.
   * @param version
   *          the information object type version.
   *
   * @return <b>PublisherSequence </b>- the sequence created that will be characterized and activated to publish information objects
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   * @throws UnsupportedVersionException
   *
   * @see PublisherSequence
   * @see InfoObjectSequence
   *
   * @since CAPI 1.0
   */
  public org.infospherics.jbi.client.PublisherSequence createPublisherSequence(String typeName, String version)
      throws PermissionDeniedException, PlatformFailureException, UnsupportedVersionException
  {
      org.infospherics.jbi.client.PublisherSequence myPublisherSequence = _connection.createPublisherSequence(typeName, version);

      if (_relationMonitorService != null)
      {
          String channelId = null;

          try
          {
              channelId = ((mil.af.rl.im.capi.client.impl.PublisherSequence)myPublisherSequence).getSequenceUID();
          }
          catch (StaleSequenceUIDException e)
          {
              e.printStackTrace();
          }
          catch (UnimplementedException e)
          {
            e.printStackTrace();
          }

          String infoObjectType = myPublisherSequence.getInfoObjectTypeName();
          if (infoObjectType == null)
              infoObjectType = "";

          String producerId = myAgentConfig.getAgentId();

          if ((producerId != null) || (channelId != null))
              _relationMonitorService.reportProducerChannel(producerId, channelId, infoObjectType);
      }

	  return myPublisherSequence;
  }

  /**
   * Creates a SubscriberSequence that will subsequently be used to subscribe for information objects published to the JBI platform. The sequence is
   * created identifying the specific information object type that will be used as the first level information object filter. Unique type name and
   * version strings are supplied to identify the information object type. A schema must exist in the metadata schema repository that corresponds to
   * the declared information object type.
   *
   * @param typeName
   *          the information object type name.
   * @param version
   *          the information object type version.
   *
   * @return <b>SubscriberSequence </b>- the sequence created that will be characterized and activated to subscribe to the specified information
   *         object type
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   * @throws UnsupportedVersionException
   *
   * @see SubscriberSequence
   * @see InfoObjectSequence
   *
   * @since CAPI 1.0
   */
  public org.infospherics.jbi.client.SubscriberSequence createSubscriberSequence(String typeName, String version)
      throws PermissionDeniedException, PlatformFailureException, UnsupportedVersionException
  {
      org.infospherics.jbi.client.SubscriberSequence mySubscriberSequence = _connection.createSubscriberSequence(typeName, version);

      if (_relationMonitorService != null)
      {
          String channelId = null;

          try
          {
              channelId = ((mil.af.rl.im.capi.client.impl.SubscriberSequence)mySubscriberSequence).getSequenceUID();
          }
          catch (StaleSequenceUIDException e)
          {
              e.printStackTrace();
          }
          catch (UnimplementedException e)
          {
            e.printStackTrace();
          }

          String infoObjectType = mySubscriberSequence.getInfoObjectTypeName();
          if (infoObjectType == null)
              infoObjectType = "";

          String consumerId = myAgentConfig.getAgentId();

          if ((consumerId != null) || (channelId != null))
              _relationMonitorService.reportConsumerChannel(consumerId, channelId, infoObjectType);
      }

	  return mySubscriberSequence;
  }

  /**
   * Creates a QuerySequence that will subsequently be used to submit queries to to the JBI platform. The sequence is created identifying the specific
   * information object type that will be used as the first level information object filter. Unique type name and version strings are supplied to
   * identify the information object type. A schema must exist in the metadata schema repository that corresponds to the declared information object type.
   *
   * @param typeName
   *          is the information object type name.
   * @param version
   *          is the information object type version.
   *
   * @return <b>QuerySequence </b>- the sequence created that will be characterized and activated to query the specified information object type
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   * @throws UnsupportedVersionException
   *
   * @see QuerySequence
   * @see InfoObjectSequence
   *
   * @since CAPI 1.0
   */
  public org.infospherics.jbi.client.QuerySequence createQuerySequence(String typeName, String version)
      throws PermissionDeniedException, PlatformFailureException, UnsupportedVersionException
  {
	  return _connection.createQuerySequence(typeName, version);
  }

  /**
   * This method will destroy the specified sequence that was previously created by the client. The sequence can be deleted anytime however the
   * intention is to destroy the sequence after activation when the information exchange is complete to free platform and client resources.
   *
   * @param sequence
   *          the sequence to be destroyed
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see PublisherSequence
   * @see SubscriberSequence
   * @see QuerySequence
   * @see InfoObjectSequence
   *
   * @since CAPI 1.0
   */
  public void destroySequence(InfoObjectSequence sequence) throws PermissionDeniedException, PlatformFailureException
  {
	  _connection.destroySequence(sequence);
  }

  /**
   * This method creates an information object that contains the information object which is the unit of information managed within a JBI platform.
   * Information objects are created by publishers (clients) disseminating information. The Information Object is comprised of three parts:
   * <ul>
   * <li>Type specification - the information type comprised of a type name and version identifier and is used as the first level matching of the
   * information object for future subscriptions or queries.
   * <li>Metadata - an XML document the contains the �information about the payload� that describes the payload information content and allows for
   * the second level matching of the information object for future subscriptions or queries.
   * <li>Payload - the information that is being distributed. The data in the payload can be represented in any format, in so far as the information
   * format is finite. This basically includes any text, graphical or binary format. The payload may be null which assumes that the metadata contains
   * all the relevant data needed for this information object.
   * </ul>
   *
   * @param typeName
   *          the information object type name.
   * @param version
   *          the information object version number.
   * @param payload
   *          the actual payload that the application wishes to place within the container (may be null).
   * @param metadataXML
   *          the information object specific metadata that is used to describe the payload.
   *
   * @return <b>InfoObject </b> the newly created information object container
   *
   * @throws InvalidPayloadException
   * @throws InvalidMetadataException
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   * @throws UnsupportedVersionException
   * @throws ObjectSizeException
   * @throws VersionNumberException
   *
   * @see InfoObject
   *
   * @since CAPI 1.0 <br>
   *        CAPI 1.5: Modified for byte[] payload instead of serializable
   */
  public InfoObject createInfoObject(String typeName, String version, byte[] payload, String metadataXML)
      throws InvalidPayloadException, InvalidMetadataException, PermissionDeniedException, PlatformFailureException, UnsupportedVersionException,
          ObjectSizeException, VersionNumberException
  {
	  return _connection.createInfoObject(typeName, version, payload, metadataXML);
  }

    public InfoObject createInfoObjectByRef(String string, String string1, String string2) throws InvalidMetadataException, PermissionDeniedException, PlatformFailureException, UnsupportedVersionException, VersionNumberException
    {
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
   * This method destroys an information object specified by a given UID. The method would primarily be used by the information management staff to
   * remove information objects based on the specific platform policy. For example, the information object temporal extent has expired and the IM
   * staff purges the information repository of all expired objects. Specific platform privileges would be necessary for a given user to have
   * permission to successfully use this method.
   *
   * @param infoObjectUID
   *          the UID of the information object to be destroyed
   *
   * @return <b>InfoObject </b> a copy of the information object container just destroyed
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see InfoObject
   *
   * @since CAPI 1.0
   */
  public InfoObject destroyInfoObject(String infoObjectUID) throws PermissionDeniedException, PlatformFailureException
  {
	  return _connection.destroyInfoObject(infoObjectUID);
  }

  /**
   * This method is used to create a TypeManagement interface to the Metadata Schema Repository for the specific platform. The Metadata Schema
   * Repository serves as a repository for the definition of the information object types and schemas. <br>
   * This interface can be used for subsequent creation and retrieval of:
   * <ul>
   * <li>Information Object Descriptors
   * <li>Attribute Descriptors
   * <li>Information Object Types
   * <li>Parent - Child Type Relationships
   * </ul>
   *
   * @return <b>MetadataRepository </b> a ref used to subsequently interract with the metadata schema repository
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see MetadataRepository
   *
   * @since CAPI 1.0
   */
  public org.infospherics.jbi.client.typemgt.MetadataRepository getMetadataRepository( )
      throws PlatformFailureException, PermissionDeniedException
  {
	  return _connection.getMetadataRepository();
  }

  /**
   * This method is used to provide the array of all publisher sequences previously created and associated with the client.
   *
   * @return <b>PublisherSequence[] </b> the array of publisher sequences associated with the client
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see PublisherSequence
   *
   * @since CAPI 1.0
   */
  public PublisherSequence[] getPublisherSequences( ) throws PlatformFailureException, PermissionDeniedException
  {
	  return _connection.getPublisherSequences();
  }

  /**
   * This method is used to provide the array of all subscriber sequences previously created and associated with the client.
   *
   * @return <b>SubscriberSequence[] </b> the array of subscriber sequences associated with the client
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see SubscriberSequence
   *
   * @since CAPI 1.0
   */
  public SubscriberSequence[] getSubscriberSequences( ) throws PlatformFailureException, PermissionDeniedException
  {
	  return _connection.getSubscriberSequences();
  }

  /**
   * This method is used to provide the array of all query sequences previously created and associated with the client.
   *
   * @return <b>QuerySequence[] </b> the array of query sequences associated with the client
   *
   * @throws PermissionDeniedException
   * @throws PlatformFailureException
   *
   * @see QuerySequence
   *
   * @since CAPI 1.0
   */
	public QuerySequence[] getQuerySequences() throws PlatformFailureException, PermissionDeniedException
	{
		return _connection.getQuerySequences();
	}

	// -------- KAoS methods

	private String getUserId(String clientRep)
	{
		int indx = clientRep.indexOf("<username>");

		if (indx > 0)
			clientRep = clientRep.substring(indx + 10);

		indx = clientRep.indexOf("</username>");

		String id = clientRep.substring(0, indx);
		System.out.println("id: " + id);

		return id;
	}

	private void registerWithKAoS()
	{
		Registration registration = null;

		CSIFactory.setCurrentTransport("tcp");

		try
        {
			registration = CSIFactory.getRegistration();
		}
		catch(Exception e)
        {
			e.printStackTrace();
			System.out.println("\n\n\t*** UNABLE TO CONNECT TO KAoS!  Will continue without KAoS services!");

			return;
		}

        myAgentConfig = registration.getAgentConfiguration();

        // Try to register domains defined in agentConfig.cfg file.


        List domainNames = myAgentConfig.getDomains();

        for (Iterator domainsIter = domainNames.iterator(); domainsIter.hasNext(); )
        {
            String dName  = (String) domainsIter.next();
            System.out.println("\n Will register domain: " + dName + "\n");

            try
            {
            	registration.registerDomain(dName);
            }
            catch (DirectoryFailure de)
            {
                System.out.println("\n\n\t*** Due to Directory Failure unable to register domain: " + dName + "\n\n\t*** Will continue without KAoS services!");
                de.printStackTrace();

                return;
            }
            catch(AlreadyRegisteredException ae)
            {
                System.out.println("\n\t*** The following domain has already been registered: " + dName);
            }
        }

		myAgentDesc =
            registration.createKAoSAgentDescription(myAgentConfig.getAgentId(), myAgentConfig.getAgentId(), (Vector)domainNames, "tcp", null);
//		agentDescription.setEntityOntologicalTypes(agentConfig.getOntologicalTypes());

		System.out.println("\n\tKAoSWrappedConnection.registerWithKAoS: will register in KAoS using KAoSAgentDescription:\n " + myAgentDesc);

		try
        {
			registration.registerEntity(myAgentDesc, true, false);    // guarded and replace registration
		}
		catch(DirectoryFailure df)
        {
			df.printStackTrace();
			System.out.println("\n\n\tDue to Directory Failure unable to register client: " + myAgentConfig.getAgentId() + "\n\n\t*** Will continue without KAoS services!");

		}
		catch(AlreadyRegisteredException are)
        {
			are.printStackTrace();
			System.out.println("\n\n\tClient: " + myAgentConfig.getAgentId() + " has been already registered with KAoS!");

		}
		catch(GuardInstantiationException gie)
        {
			gie.printStackTrace();
			System.out.println("\n\n\tDue to Guard instantiation failure unable to register client: " + myAgentConfig.getAgentId() + "\n\n\t*** Will continue without KAoS services!");
		}
	}
}
