package coi.jbi.client;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.tree.DefaultElement;
import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.exception.ObjectUnavailableException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.TimeoutException;
import org.infospherics.jbi.client.exception.UnimplementedException;

import coi.jbi.demo.metoc.OntologyConcepts;

public class InfoObjectHelper
{
    private InfoObject _infoObj = null;
    private Document _metaDoc = null;
    private Document _extMetaDoc = null;
    private HashMap _namespaces = null;
    private String _extMetadataXML = null;
    private String _metadataXML = null;

    public static final String imNamespacePrefix = "im";
    public static final String sysNamespacePrefix = "sys";
    public static final String xsiNamespacePrefix = "xsi";

    public static final String imNamespaceURI = "http://jbischemas.rl.af.mil/schemas/im";
    public static final String sysNamespaceURI = "http://jbischemas.rl.af.mil/schemas/sys";
    public static final String xsiNamespaceURI = "http://www.w3.org/2001/XMLSchema-instance";

    public static final String rdfNamespacePrefix = "rdf";
    public static final String rdfNamespaceURI ="http://www.w3.org/1999/02/22-rdf-syntax-ns#";
    public static final String rdfsNamespacePrefix = "rdfs";
    public static final String rdfsNamespaceURI ="http://www.w3.org/2000/01/rdf-schema#";
    public static final String owlNamespacePrefix = "owl";
    public static final String owlNamespaceURI ="http://www.w3.org/2002/07/owl#";
    public static final String damlNamespacePrefix = "daml";
    public static final String damlNamespaceURI ="http://www.daml.org/2001/03/daml+oil#";

    public static final String xpathName = "//" + imNamespacePrefix + ":Name";
    public static final String xpathMajorVersion = "//" + imNamespacePrefix + ":MajorVersion";
    public static final String xpathMinorVersion = "//" + imNamespacePrefix + ":MinorVersion";
    public static final String xpathPublicationTime = "//" + imNamespacePrefix + ":PublicationTime";
    public static final String xpathInfoObjectID = "//" + imNamespacePrefix + ":InfoObjectID";
    public static final String xpathPublisherID = "//" + imNamespacePrefix + ":PublisherID";
    public static final String xpathPlatformID = "//" + imNamespacePrefix + ":PlatformID";

    public InfoObjectHelper(InfoObject infoObj) throws ObjectUnavailableException, PlatformFailureException, TimeoutException, UnimplementedException, DocumentException
    {
        _infoObj = infoObj;

        _namespaces = new HashMap();
        // meta namespaces
        _namespaces.put(xsiNamespacePrefix, xsiNamespaceURI);
        //ext meta namespaces
        _namespaces.put(imNamespacePrefix, imNamespaceURI);
        _namespaces.put(sysNamespacePrefix, sysNamespaceURI);
        _namespaces.put(OntologyConcepts.WEATHER_PROPERTIES_NSPREFIX, OntologyConcepts.WEATHER_PROPERTIES_NAMESPACE);
        _namespaces.put(OntologyConcepts.WEATHER_PUBLICATIONS_NSPREFIX, OntologyConcepts.WEATHER_PUBLICATIONS_NAMESPACE);

        _metadataXML = infoObj.getMetadata();
        // for unknown reasons, the '&amp;' has been decoded to '&' but only in the xmlns declarations
        _metadataXML = _metadataXML.replaceAll("&partName=", "&amp;partName=");
        // for unknown reasons, the jbi server requires all ampersands to be doubly-encoded as '&amp;amp;'
        _metadataXML = _metadataXML.replaceAll("&amp;amp;", "&amp;");
        _metaDoc = DocumentHelper.parseText(_metadataXML);

        _extMetadataXML = infoObj.getExtendedMetadata();
        _extMetadataXML = _extMetadataXML.replaceAll("&partName=", "&amp;partName=");
        _extMetadataXML = _extMetadataXML.replaceAll("&amp;amp;", "&amp;");
        _extMetaDoc = DocumentHelper.parseText(_extMetadataXML);
    }

    public String getMetadata()
    {
        return _metadataXML;
    }

    public String getExtMetadata()
    {
        return _extMetadataXML;
    }

    private String getNodeValue(String xpathString, Document doc)
    {
        XPath xp = DocumentHelper.createXPath(xpathString);
        xp.setNamespaceURIs(_namespaces);

        Node node = xp.selectSingleNode(doc);
        String val = null;
        if (node != null)
        {
            val = node.getText();
        }
        return val;
    }

    //
    // org.infospherics.jbi.client.InfoObject Properties
    //
    public String getOntClassURI() throws ObjectUnavailableException
    {
        XPath xp = DocumentHelper.createXPath("/child::node()/child::node()");
        xp.setNamespaceURIs(_namespaces);

        List nodeList = xp.selectNodes(_metaDoc);
        if (nodeList == null || nodeList.size() == 0)
        {
            throw new ObjectUnavailableException("No OWL individuals found in metadata.");
        }
        DefaultElement lastElementTypeNode = null;
        Iterator iter = nodeList.iterator();
        while(iter.hasNext())
        {
            Node node = (Node)iter.next();
            if (node.getNodeType() == Node.ELEMENT_NODE)
            {
                lastElementTypeNode = (DefaultElement)node;
            }
        }
        String val = null;
        if (lastElementTypeNode != null)
        {
            val = lastElementTypeNode.getNamespaceURI() + lastElementTypeNode.getName();
        }
        return val;
    }

    public String getTypeName()
    {
        return _infoObj.getTypeName();
    }

    public String getUID()
    {
        return _infoObj.getUID();
    }

    public String getVersion()
    {
        return _infoObj.getVersion();
    }

    public byte[] getPayload() throws ObjectUnavailableException, PlatformFailureException, TimeoutException
    {
        return _infoObj.getPayload();
    }

    //
    // XML Extended Metadata Properties
    //

    public String getName()
    {
        return getNodeValue(xpathName, _extMetaDoc);
    }

    public String getMajorVersion()
    {
        return getNodeValue(xpathMajorVersion, _extMetaDoc);
    }

    public String getMinorVersion()
    {
        return getNodeValue(xpathMinorVersion, _extMetaDoc);
    }

    public String getPublicationTime()
    {
        return getNodeValue(xpathPublicationTime, _extMetaDoc);
    }

    public String getInfoObjectID()
    {
        return getNodeValue(xpathInfoObjectID, _extMetaDoc);
    }

    public String getPublisherID()
    {
        return getNodeValue(xpathPublisherID, _extMetaDoc);
    }

    public String getPlatformID()
    {
        return getNodeValue(xpathPlatformID, _extMetaDoc);
    }
}