package coi.jbi.client;

import mil.af.rl.im.capi.client.util.Logger;
import mil.af.rl.im.capi.client.util.Utils;
import mil.af.rl.im.capi.client.util.gui.ConnectionDescriptorDialog;

import org.infospherics.jbi.client.PublisherSequence;
import org.infospherics.jbi.client.SubscriberSequence;
import org.infospherics.jbi.client.exception.ConnectionException;
import org.infospherics.jbi.client.exception.InvalidPredicateException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.PredicateLanguageException;
import org.infospherics.jbi.client.exception.SequenceStateException;
import org.infospherics.jbi.client.exception.UnrecognizedObjectTypeException;
import org.infospherics.jbi.client.exception.UnsupportedVersionException;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.registration.AgentConfiguration;
import kaos.core.csi.registration.Registration;

public class JBIBaseClient
{
    private static String JBI_SERVER_PASSWORD = "test";
    
    protected String _clientName = null;
    
    protected org.infospherics.jbi.client.ConnectionManager _manager = null;
    protected org.infospherics.jbi.client.Connection _connection = null;
    protected String _connectionDescriptor = null;

    public JBIBaseClient()
    {
        Registration registration = null;

        CSIFactory.setCurrentTransport("tcp");

        try
        {
            // Register in KAoS
            registration = CSIFactory.getRegistration();
            AgentConfiguration agentConfig = registration.getAgentConfiguration(); // Obtain client bootstrap information e.g. _clientName
            _clientName = agentConfig.getAgentId();
            
            int endNamespace = _clientName.indexOf('#');
            if (endNamespace >= 0)
                _clientName = _clientName.substring(endNamespace + 1);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            System.out.println("\n\n\t*** UNABLE TO CONNECT TO KAoS!  Will continue without KAoS services!");

            return;
        }
        
        // Add shutdonw hook
        Runtime.getRuntime().addShutdownHook(new Thread()
        {
            public void run() 
            {
                disconnect();
            }
        });
    }

    public JBIBaseClient(String connectionDescriptor)
    {
        this();
        _connectionDescriptor = connectionDescriptor;
    }

    // Methods creating connection with JBI Server
    public void connect() throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {
        connect(_clientName, _connectionDescriptor);
    }

    private void connect(String clientName, String connectionDescriptor) throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {
        // Obtain JBI Server connection info
        String fs = System.getProperty("file.separator");
        String clientConfigPath = System.getProperty("jbi.client.properties.dir");

        if (connectionDescriptor == null)
            connectionDescriptor = Utils.readFile(clientConfigPath + fs + "ConnectionDescriptor.xml");
        
        if (connectionDescriptor == null)
            connectionDescriptor = getConnectionDescriptorFromDialog();

        _manager = new ConnectionManager();

        // Create a connection to the JBI platform. 
        // The method returns an array of the fully populated ConnectionDescriptors of the platforms 
        // that match the supplied discovery criteria.
        String[] matchedPlatforms = _manager.getPlatforms(connectionDescriptor);

        System.out.println("\n\nPresenting Discovered Platforms That Match Search Criteria: ");

        if (matchedPlatforms != null && matchedPlatforms.length > 0)
        {
            for (int i=0; i<matchedPlatforms.length; i++)
                System.out.println("\n\n MatchedPlatform " + i + ": " + matchedPlatforms[i]);
        }
        else
        {
            System.out.println("Sorry there are no platforms that match your platform descriptor search criteria..." + "\nExiting!");
            System.exit(-1);
        }

        // Selecgt first avialable JBI server
       _connection = _manager.createConnection(clientName, matchedPlatforms[0]);

       // Dynamically create JBI credentials
       String credentials = 
           "<?xml version=\"1.0\" encoding=\"UTF-8\"?><credentials><username>" +
           clientName + "</username><password>" +
           JBI_SERVER_PASSWORD + "</password></credentials>";

       // Authenticate in JBI using a specific role (user and role).
       _connection.authenticate(credentials);

       // Connect to JBI
       _connection.connect();

       try
       {
           // Sleep to allow time to register for channel creation events
           Thread.sleep(5000); 
       }
       catch(InterruptedException e)
       {
           e.printStackTrace();
       }

       System.out.println("\nMy connection uid: " + _connection.getConnectionUID());
    }

    public void disconnect()
    {
        if (_connection != null)
        {
            try 
            {
                System.out.println("Disconnecting from JBI: " + _connection.getConnectionName());
                _connection.disconnect();
            }
            catch (Exception e) 
            {
                System.out.println("Exception on destroyConnection! " + e.getMessage() + "  " + e.getLocalizedMessage());
                e.printStackTrace();
            }
            try 
            {
                _manager.destroyConnection(_connection);
            }
            catch (Exception e) 
            {
                System.out.println("Exception on destroyConnection! " + e.getMessage() + "  " + e.getLocalizedMessage());
                e.printStackTrace();
            }
        }
    }

    // Method creating the Publish sequance
    public PublisherSequence initPublisherChannel(String jbiPackageType, String jbiVersion) 
        throws PermissionDeniedException, PlatformFailureException, UnsupportedVersionException, SequenceStateException
    {
        // Publisher Sequence refs.
        PublisherSequence basicPub = null;

        // Create sequences based on information object type and type version.
        basicPub = getConnection().createPublisherSequence(jbiPackageType, jbiVersion);

        // Archival related attribute
        //    String archive = Utils.readFile(clientConfigPath + fs + "infoObject" + fs + "Archival.xml");

        // Set sequance attribute
        //    basicPub.setSequenceAttribute("Archival", "1", archive);

        return basicPub;
    }

    // Method creating subscribe sequance
    public SubscriberSequence initSubscriberChannel(String jbiPackageType, String jbiVersion, String xpathMetaFilter) 
        throws PermissionDeniedException, PlatformFailureException, UnsupportedVersionException, UnrecognizedObjectTypeException, 
               SequenceStateException, InvalidPredicateException, PredicateLanguageException
    {
        // Subscriber Sequence refs.
        SubscriberSequence basicSub = null;

        // XPath predicate - generic to match any published metadata
        if (xpathMetaFilter == null || xpathMetaFilter.length() == 0)
        {
            xpathMetaFilter = "(//node())";
        }
        String predicate = Utils.createPredicate("TestPredicate", "XPath", xpathMetaFilter);
        Logger.log("predicateOne: " + predicate + "\n\n", false, true, 0);

        // Create sequences based on information object type and type version.
        basicSub = getConnection().createSubscriberSequence(jbiPackageType, jbiVersion);

        // Set the predicate that will be applied to the published metadata on behalf of this subscriber.
        basicSub.setSequencePredicate(predicate);

        return basicSub;
    }

    // ---- Auxiliary methods ----
    
    public org.infospherics.jbi.client.ConnectionManager getConnectionManager()
    {
        return _manager;
    }

    public org.infospherics.jbi.client.Connection getConnection()
    {
        return _connection;
    }

    public String getClientName()
    {
        return _clientName;
    }

    public String getConnectionDescriptor()
    {
        return _connectionDescriptor;
    }

    public String getConnectionDescriptorFromDialog()
    {
       ConnectionDescriptorDialog cdd = new ConnectionDescriptorDialog();
       String connectionDescriptor = cdd.showPassDialog();
       return connectionDescriptor;
    }
}